/*-
 * Copyright (c) 1995 by Kenji Rikitake. All rights reserved.
 * Redistribution and usage conditions are mentioned below in
 * the copyright statement of The Regents of the University of California.
 *
 * Kenji Rikitake $Id: mailfeed-fwd.c,v 1.3 1995/10/15 12:13:25 kenji Exp $
 */	
/*-
 * Some parts of this source code have been derived from rmail.c
 * which is distributed with 4.4BSD and Eric Allman's sendmail 8.x.
 * The SCCS ID of the latest rmail.c is 
 * "@(#)rmail.c   8.3 (Berkeley) 5/15/95".
 */
/*-
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char MAILFEED_Copyright[] =
"Copyright (c) 1995 by Kenji Rikitake. All rights reserved.\n";
static char UCB_copyright[] =
"@(#) Copyright (c) 1988, 1993\n\
	The Regents of the University of California.  All rights reserved.\n";
#endif /* not lint */

/*
 * mailfeed-fwd -- sendmail local delivery agent for MAILFEED 
 *
 * This program is called from sendmail as a delivery agent, with
 * following mailer configuration in sendmail.cf:
 *
 * Mmailfeed, P=/usr/local/bin/mailfeed-fwd, F=lmDFMun, S=11, R=21,
 * A=mailfeed-fwd $h $g $u
 *
 * usage: mailfeed-fwd <local-mbox ($h)> <from-addr ($g)> <recipients ($u)>...
 * 
 * This program adds following RFC822-like header lines to the top of 
 * the message read from the standard input up to an EOF, 
 * and forwards the modified message to the local mailbox delivery agent
 * (e.g., mail.local).
 * 
 *	X-Mailfeed-Sender: <address of the sender>
 *	X-Mailfeed-Recipient-Number: <number of recipients by decimal>
 *	X-Mailfeed-Recipient-Address: <address of the first recipient>
 *	...
 *	X-Mailfeed-Recipient-Address: <address of the last recipient>
 */

#include <sys/types.h>
#include <sys/wait.h>

#include <errno.h>
#include <stdio.h>
#include <sysexits.h>
#include <unistd.h>

#include "mailfeed.h"

#define _PATH_MAILLOCAL "/usr/libexec/mail.local" 
#define PROGRAM_NAME "mailfeed-fwd"

void err __P((int, const char *, ...));
void usage __P((void));

int main(argc, argv)
	int argc;
	char *argv[];
{
	int i, pid, status;
	int pdes[2];
	FILE *fp;
	char lbuf[2048];

	if (argc < 4) {
		usage();
	}

	/* open pipe for local mailer */
	if (pipe(pdes) < 0) {
		err(EX_OSERR, NULL);
	}

	switch (pid = vfork()) {
	case -1:	/* Error */
		err(EX_OSERR, NULL);
	case 0:		/* Child */
		if (pdes[0] != STDIN_FILENO) {
			(void)dup2(pdes[0], STDIN_FILENO);
			(void)close(pdes[0]);
		}
		(void)close(pdes[1]);
		/* mail.local -r <from-addr> -d <local-mbox> */
		execl(_PATH_MAILLOCAL, _PATH_MAILLOCAL,
			"-r", argv[2], "-d", argv[1], NULL);
		_exit(127);
		/* NOTREACHED */
	}

	if ((fp = fdopen(pdes[1], "w")) == NULL) {
		err(EX_OSERR, NULL);
	}
	(void)close(pdes[0]);
	
	/* 
	 * adding MAILFEED headers
	 */

	(void)fprintf(fp, "%s %s\n", HEADER_SENDER, argv[2]);
	(void)fprintf(fp, "%s %d\n", HEADER_REC_NUM, argc - 3);
	for (i = 3; i < argc; i++) {
		(void)fprintf(fp, "%s %s\n", HEADER_REC_ADDR, argv[i]);
	}

	/* Copy the file to the pipe until the EOF */
	while (fgets(lbuf, sizeof(lbuf), stdin) != NULL) {
		(void)fprintf(fp, "%s", lbuf);
	}

	if (ferror(stdin)) {
		err(EX_TEMPFAIL, "stdin: %s", strerror(errno));
	}

	if (fclose(fp)) {
		err(EX_OSERR, NULL);
	}

	if ((waitpid(pid, &status, 0)) == -1) {
		err(EX_OSERR, "%s", _PATH_MAILLOCAL);
	}

	if (!WIFEXITED(status)) {
		err(EX_OSERR,
		    "%s: did not normally terminate", _PATH_MAILLOCAL);
	}

	if (WEXITSTATUS(status)) {
		err(status, "%s: terminated with %d (non-zero) status",
		    _PATH_MAILLOCAL, WEXITSTATUS(status));
	}	

	exit(EX_OK);
}

void
usage()
{
	(void)fprintf(stderr, 
		      "usage: %s local-mbox from-addr recipients ...\n",
		      PROGRAM_NAME);
	exit(EX_USAGE);
}

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

void
#ifdef __STDC__
err(int eval, const char *fmt, ...)
#else
err(eval, fmt, va_alist)
	int eval;
	const char *fmt;
	va_dcl
#endif
{
	va_list ap;
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	(void)fprintf(stderr, "%s: ", PROGRAM_NAME);
	(void)vfprintf(stderr, fmt, ap);
	va_end(ap);
	(void)fprintf(stderr, "\n");
	exit(eval);
}
